@@ -41,11 +41,13 @@
 #define LICENSE "This program comes with ABSOLUTELY NO WARRANTY.\nLicensed under the GNU Public License version 3 or later.\n"
 
 
-const char *peng_version = "4.01.0078"; /* CHANGEME */
+const char *peng_version = "4.01.0082"; /* CHANGEME */
 
 
 #define MAXFNLEN              1024
 #define MIN_LOCRR_SEQ_LEN        8
+#define EXTRAHEADERSIZE0         0
+#define EXTRAHEADERSIZE          0
 
 
 struct easyset
@@ -138,6 +140,13 @@ uint32_t *parseeasy(char *s, struct easyset es[])
 }
 
 
+void progressmeter(uint64_t p, uint64_t m)
+{
+    printf("%6.2f%% done\r", (double)p/(double)m*1e2);
+    fflush(stdout);
+}
+
+
 int xmain(int argc, char **argv)
 {
     int i,r,h1,h2;
@@ -253,7 +262,8 @@ int xmain(int argc, char **argv)
     FREE(binparm);
     
     if(!passphrase)
-        passphrase = getpass("PENG Password: ");
+        passphrase = mygetpass("PENG Password: ", eflag?"Please repeat: ":NULL, eflag?"Password mismatch.":NULL);
+        
     
     if(verbosity>2)
     {
@@ -333,30 +343,46 @@ int xmain(int argc, char **argv)
             mypce.htrx0.blksize = blksize;
             mypce.htrx0.rounds = rounds;
             mypce.htrx0.variations = variations;
-            mypce.htrx0.extra = 0;
-            r = peng_preliminary_header_write_convenience(&mypce, h2);
+            mypce.htrx0.extra0 = EXTRAHEADERSIZE0;
+            r = peng_unencrypted_header_write_convenience(&mypce, h2);
         }
         else
         {
-            r = peng_preliminary_header_read_convenience(&mypce, h1);
+            r = peng_unencrypted_header_read_convenience(&mypce, h1);
             blksize = mypce.htrx0.blksize;
             rounds = mypce.htrx0.rounds;
             variations = mypce.htrx0.variations;
             /* total -= mypce.off; */
         }
-        switch(r)
+        if(r!=0)
         {
-            case ERROR_SYSTEM_INFILE:
-                perror(infn);
-                break;
-            case ERROR_SYSTEM_OUTFILE:
-                perror(outfn);
-                break;
+            switch(r)
+            {
+                case ERROR_SYSTEM_INFILE:
+                    perror(infn);
+                    break;
+                case ERROR_SYSTEM_OUTFILE:
+                    perror(outfn);
+                    break;
+                case ERROR_MAGIC:
+                    fprintf(stderr, "%s: bad magic\n", infn);
+                    break;
+                default:
+                    fprintf(stderr, "unspecified or undefined error %d\n", r);
+            }
+            return 10;
         }
         /* printf("DEBUG: infile pos = %lu,%lu\n", (unsigned long)lseek(h1, 0, SEEK_CUR), (unsigned long)lseek(h2, 0, SEEK_CUR)); */
 #else
         /* mypce.off = 0; */
 #endif
+        
+        if(blksize < MINIMUM_BLOCKSIZE)
+        {
+            fprintf(stderr, "error: minimum blocksize is: %u\n", (unsigned)MINIMUM_BLOCKSIZE);
+            return 5;
+        }
+
         peng_cmd_prep(&mypce, blksize, rounds, variations, passphrase, eflag);
     
         if(verbosity>0)
@@ -367,10 +393,10 @@ int xmain(int argc, char **argv)
 
         if(verbosity>0)
         {
-            printf("%30s   -%c->    %-30s\n", infn, eflag ? 'e':'d', outfn);
+            printf("%30s   -%c->    %-30s\n", infn, eflag ? 'e':'d', outfn);   /* 72 chars */
             fflush(stdout);
         }
-        r = peng_cmd_process(&mypce, infn, h1, outfn, h2, multithreading, MIN_LOCRR_SEQ_LEN);
+        r = peng_cmd_process(&mypce, infn, h1, outfn, h2, multithreading, MIN_LOCRR_SEQ_LEN, (verbosity>1)?progressmeter:NULL, EXTRAHEADERSIZE);
         
         close(h1);
         close(h2);
@@ -394,6 +420,8 @@ int xmain(int argc, char **argv)
                 case ERROR_COMPAT:
                     fprintf(stderr, "%s: compatibility (version or capabilities) failure\n", argv[i]);
                     break;
+                default:
+                    fprintf(stderr, "unspecified or undefined error %d\n", r);
             }
             return 10;
         }
