@@ -1,15 +1,16 @@
 #! /bin/bash
 
-ARC="$1"
+TMP=/tmp
+ARC="`pwd`/$1"
 DIR="`basename $ARC .tar.bz2`"
 shift
 
 GPGKEY=0D1099BA
 
-mkdir $DIR
-tar -cf - "$@" |tar -xf - -C $DIR
-tar -cjf $ARC $DIR
-rm -fr $DIR
+mkdir -p $TMP/$DIR
+tar -cf - "$@" |tar -xf - -C $TMP/$DIR
+tar -cjf $ARC -C $TMP $DIR
+rm -fr $TMP/$DIR
 
 if [ "$USER" = "kj" ]
 then
